@@ -180,8 +180,6 @@ function config_trove_apache_wsgi {
         s|%APIWORKERS%|${API_WORKERS}|g;
     " -i ${trove_apache_conf}
     enable_apache_site trove-api
-    tail_log trove-access /var/log/${APACHE_NAME}/trove-api-access.log
-    tail_log trove-api /var/log/${APACHE_NAME}/trove-api.log
 }
 
 # configure_trove() - Set config files, create data dirs, etc
@@ -476,12 +474,17 @@ function create_guest_image {
     glance_image_id=$(openstack --os-region-name RegionOne --os-password ${SERVICE_PASSWORD} \
       --os-project-name service --os-username trove \
       image create ${image_name} \
-      --disk-format qcow2 --container-format bare --property hw_rng_model='virtio' --file ${image_file} \
+      --disk-format qcow2 --container-format bare \
+      --tag trove \
+      --property hw_rng_model='virtio' \
+      --file ${image_file} \
+      --debug \
       -c id -f value)
+     echo "Glance image ${glance_image_id} uploaded"
 
     echo "Register the image in datastore"
     $TROVE_MANAGE datastore_update $TROVE_DATASTORE_TYPE ""
-    $TROVE_MANAGE datastore_version_update $TROVE_DATASTORE_TYPE $TROVE_DATASTORE_VERSION $TROVE_DATASTORE_TYPE $glance_image_id "" 1
+    $TROVE_MANAGE datastore_version_update $TROVE_DATASTORE_TYPE $TROVE_DATASTORE_VERSION $TROVE_DATASTORE_TYPE "" "" 1 --image-tags trove
     $TROVE_MANAGE datastore_update $TROVE_DATASTORE_TYPE $TROVE_DATASTORE_VERSION
 
     echo "Add parameter validation rules if available"
