@@ -17,12 +17,15 @@
 
 import os.path
 
+from keystoneauth1 import loading
+from keystonemiddleware import auth_token
 from oslo_config import cfg
-from oslo_config.cfg import NoSuchOptError
 from oslo_config import types
+from oslo_config.cfg import NoSuchOptError
 from oslo_log import log as logging
 from oslo_middleware import cors
 from osprofiler import opts as profiler
+from oslo_log import versionutils
 
 from trove.common.i18n import _
 from trove.version import version_info as version
@@ -91,7 +94,7 @@ common_opts = [
     cfg.BoolOpt('neutron_api_insecure', default=False,
                 help="Allow to perform insecure SSL requests to neutron."),
     cfg.URIOpt('cinder_url', help='URL without the tenant segment.'),
-    cfg.StrOpt('cinder_service_type', default='volumev2',
+    cfg.StrOpt('cinder_service_type', default='volumev3',
                help='Service type to use when searching catalog.'),
     cfg.StrOpt('cinder_endpoint_type', default='publicURL',
                help='Service endpoint type to use when searching catalog.'),
@@ -175,10 +178,10 @@ common_opts = [
                help="Maximum time (in seconds) to wait for Guest Agent "
                     "'quick' requests (such as retrieving a list of "
                     "users or databases)."),
-    cfg.IntOpt('agent_call_high_timeout', default=60 * 5,
+    cfg.IntOpt('agent_call_high_timeout', default=60 * 3,
                help="Maximum time (in seconds) to wait for Guest Agent 'slow' "
                     "requests (such as restarting the database)."),
-    cfg.IntOpt('agent_replication_snapshot_timeout', default=36000,
+    cfg.IntOpt('agent_replication_snapshot_timeout', default=60 * 30,
                help='Maximum time (in seconds) to wait for taking a Guest '
                     'Agent replication snapshot.'),
     cfg.IntOpt('command_process_timeout', default=30,
@@ -186,14 +189,19 @@ common_opts = [
                     'commands to complete.'),
     # The guest_id opt definition must match the one in cmd/guest.py
     cfg.StrOpt('guest_id', default=None, help="ID of the Guest Instance."),
-    cfg.IntOpt('state_change_wait_time', default=60 * 10,
-               help='Maximum time (in seconds) to wait for a state change.'),
+    cfg.StrOpt('controller_address',
+               help='The address used to download Trove code by guest agent '
+                    'in developer mode. This address is inserted into the '
+                    'file /etc/trove/controller.conf inside the guest.'),
+    cfg.IntOpt('state_change_wait_time', default=180,
+               help='Maximum time (in seconds) to wait for database state '
+                    'change.'),
     cfg.IntOpt('state_change_poll_time', default=3,
                help='Interval between state change poll requests (seconds).'),
     cfg.IntOpt('agent_heartbeat_time', default=10,
                help='Maximum time (in seconds) for the Guest Agent to reply '
                     'to a heartbeat request.'),
-    cfg.IntOpt('agent_heartbeat_expiry', default=60,
+    cfg.IntOpt('agent_heartbeat_expiry', default=90,
                help='Time (in seconds) after which a guest is considered '
                     'unreachable'),
     cfg.IntOpt('num_tries', default=3,
@@ -213,6 +221,9 @@ common_opts = [
                default=10,
                help='Default maximum number of instances per tenant.',
                deprecated_name='max_instances_per_user'),
+    cfg.IntOpt('max_ram_per_tenant',
+               default=-1,
+               help='Default maximum total amount of RAM in MB per tenant.'),
     cfg.IntOpt('max_accepted_volume_size', default=10,
                help='Default maximum volume size (in GB) for an instance.'),
     cfg.IntOpt('max_volumes_per_tenant', default=40,
@@ -242,7 +253,10 @@ common_opts = [
                'becomes required in the instance create request.'),
     cfg.StrOpt('datastore_manager', default=None,
                help='Manager class in the Guest Agent, set up by the '
-               'Taskmanager on instance provision.'),
+                    'Taskmanager on instance provision.'),
+    cfg.StrOpt('datastore_version', default=None,
+               help='The guest datastore version that is set by the '
+                    'Taskmanager during instance provision.'),
     cfg.StrOpt('block_device_mapping', default='vdb',
                help='Block device to map onto the created instance.'),
     cfg.IntOpt('server_delete_time_out', default=60,
@@ -293,9 +307,11 @@ common_opts = [
                help='The region this service is located.'),
     cfg.StrOpt('backup_runner',
                default='trove.guestagent.backup.backup_types.InnoBackupEx',
-               help='Runner to use for backups.'),
+               help='Runner to use for backups.',
+               deprecated_for_removal=True),
     cfg.DictOpt('backup_runner_options', default={},
-                help='Additional options to be passed to the backup runner.'),
+                help='Additional options to be passed to the backup runner.',
+                deprecated_for_removal=True),
     cfg.BoolOpt('verify_swift_checksum_on_restore', default=True,
                 help='Enable verification of Swift checksum before starting '
                 'restore. Makes sure the checksum of original backup matches '
@@ -304,21 +320,45 @@ common_opts = [
                 help='Require the replica volume size to be greater than '
                 'or equal to the size of the master volume '
                 'during replica creation.'),
-    cfg.StrOpt('storage_strategy', default='SwiftStorage',
+    cfg.StrOpt('storage_strategy', default='swift',
                help="Default strategy to store backups."),
     cfg.StrOpt('storage_namespace',
                default='trove.common.strategies.storage.swift',
-               help='Namespace to load the default storage strategy from.'),
+               help='Namespace to load the default storage strategy from.',
+               deprecated_for_removal=True),
     cfg.StrOpt('backup_swift_container', default='database_backups',
                help='Swift container to put backups in.'),
     cfg.BoolOpt('backup_use_gzip_compression', default=True,
-                help='Compress backups using gzip.'),
-    cfg.BoolOpt('backup_use_openssl_encryption', default=True,
-                help='Encrypt backups using OpenSSL.'),
-    cfg.StrOpt('backup_aes_cbc_key', default='default_aes_cbc_key',
-               help='Default OpenSSL aes_cbc key.'),
-    cfg.BoolOpt('backup_use_snet', default=False,
-                help='Send backup files over snet.'),
+                help='Compress backups using gzip.',
+                deprecated_for_removal=True,
+                deprecated_since=versionutils.deprecated.VICTORIA,
+                deprecated_reason='Backup data compression is enabled by '
+                                  'default. This option is ignored.'),
+    cfg.BoolOpt(
+        'backup_use_openssl_encryption', default=True,
+        help='Encrypt backups using OpenSSL.',
+        deprecated_for_removal=True,
+        deprecated_since=versionutils.deprecated.VICTORIA,
+        deprecated_reason='Trove should not encrypt backup data on '
+                          'behalf of the user. This option is ignored.'
+    ),
+    cfg.StrOpt(
+        'backup_aes_cbc_key', default='',
+        help='Default OpenSSL aes_cbc key for decrypting backup data created '
+             'prior to Victoria.',
+        deprecated_for_removal=True,
+        deprecated_since=versionutils.deprecated.VICTORIA,
+        deprecated_reason='This option is only for backward compatibility. '
+                          'Backups created after Victoria are not encrypted '
+                          'any more.'
+    ),
+    cfg.BoolOpt(
+        'backup_use_snet', default=False,
+        help='Send backup files over snet.',
+        deprecated_for_removal=True,
+        deprecated_since=versionutils.deprecated.VICTORIA,
+        deprecated_reason='This option is not supported any more.'
+    ),
     cfg.IntOpt('backup_chunk_size', default=2 ** 16,
                help='Chunk size (in bytes) to stream to the Swift container. '
                'This should be in multiples of 128 bytes, since this is the '
@@ -373,7 +413,8 @@ common_opts = [
                          'mariadb': '7a4f82cc-10d2-4bc6-aadc-d9aacc2a3cb5'},
                 help='Unique ID to tag notification events.'),
     cfg.StrOpt('network_label_regex', default='^private$',
-               help='Regular expression to match Trove network labels.'),
+               help='Regular expression to match Trove network labels.',
+               deprecated_for_removal=True),
     cfg.StrOpt('ip_regex', default=None,
                help='List IP addresses that match this regular expression.'),
     cfg.StrOpt('black_list_regex', default=None,
@@ -428,15 +469,12 @@ common_opts = [
     cfg.IntOpt('usage_timeout', default=60 * 30,
                help='Maximum time (in seconds) to wait for a Guest to become '
                     'active.'),
-    cfg.IntOpt('restore_usage_timeout', default=36000,
+    cfg.IntOpt('restore_usage_timeout', default=60 * 60,
                help='Maximum time (in seconds) to wait for a Guest instance '
                     'restored from a backup to become active.'),
     cfg.IntOpt('cluster_usage_timeout', default=36000,
                help='Maximum time (in seconds) to wait for a cluster to '
                     'become active.'),
-    cfg.IntOpt('timeout_wait_for_service', default=120,
-               help='Maximum time (in seconds) to wait for a service to '
-                    'become alive.'),
     cfg.StrOpt('module_aes_cbc_key', default='module_aes_cbc_key',
                help='OpenSSL aes_cbc key for module encryption.'),
     cfg.ListOpt('module_types', default=['ping', 'new_relic_license'],
@@ -465,6 +503,18 @@ common_opts = [
                help='Key (OpenSSL aes_cbc) to encrypt instance keys in DB.'),
     cfg.StrOpt('instance_rpc_encr_key',
                help='Key (OpenSSL aes_cbc) for instance RPC encryption.'),
+    cfg.StrOpt('database_service_uid', default='1001',
+               help='The UID(GID) of database service user.'),
+    cfg.ListOpt('reserved_network_cidrs', default=[],
+                help='Network CIDRs reserved for Trove guest instance '
+                     'management.'),
+    cfg.BoolOpt(
+        'online_volume_resize', default=True,
+        help='If online volume resize is supported.'),
+    cfg.BoolOpt(
+        'enable_volume_az', default=False,
+        help='If true create the volume in the same availability-zone as the '
+             'instance'),
 ]
 
 
@@ -543,7 +593,7 @@ mysql_opts = [
                 help='List of UDP ports and/or port ranges to open '
                      'in the security group (only applicable '
                      'if trove_security_groups_support is True).'),
-    cfg.StrOpt('backup_strategy', default='InnoBackupEx',
+    cfg.StrOpt('backup_strategy', default='innobackupex',
                help='Default strategy to perform backups.',
                deprecated_name='backup_strategy',
                deprecated_group='DEFAULT'),
@@ -563,28 +613,10 @@ mysql_opts = [
     cfg.IntOpt('usage_timeout', default=400,
                help='Maximum time (in seconds) to wait for a Guest to become '
                     'active.'),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.mysql_impl',
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.mysql_impl',
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
     cfg.BoolOpt('volume_support', default=True,
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
-    cfg.DictOpt('backup_incremental_strategy',
-                default={'InnoBackupEx': 'InnoBackupExIncremental'},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental backup, the runner will use the default full '
-                'backup.',
-                deprecated_name='backup_incremental_strategy',
-                deprecated_group='DEFAULT'),
     cfg.StrOpt('root_controller',
                default='trove.extensions.common.service.DefaultRootController',
                help='Root controller implementation for mysql.'),
@@ -610,6 +642,16 @@ mysql_opts = [
                help='Character length of generated passwords.',
                deprecated_name='default_password_length',
                deprecated_group='DEFAULT'),
+    cfg.StrOpt(
+        'docker_image', default='mysql',
+        help='Database docker image.'
+    ),
+    cfg.StrOpt(
+        'backup_docker_image', default='openstacktrove/db-backup-mysql:1.1.0',
+        help='The docker image used for backup and restore. For mysql, '
+             'the minor version is added to the image name as a suffix before '
+             'creating container, e.g. openstacktrove/db-backup-mysql5.7:1.0.0'
+    ),
 ]
 
 # Percona
@@ -652,28 +694,10 @@ percona_opts = [
     cfg.IntOpt('usage_timeout', default=450,
                help='Maximum time (in seconds) to wait for a Guest to become '
                     'active.'),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.mysql_impl',
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.mysql_impl',
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
     cfg.BoolOpt('volume_support', default=True,
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
-    cfg.DictOpt('backup_incremental_strategy',
-                default={'InnoBackupEx': 'InnoBackupExIncremental'},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental backup, the runner will use the default full '
-                'backup.',
-                deprecated_name='backup_incremental_strategy',
-                deprecated_group='DEFAULT'),
     cfg.StrOpt('root_controller',
                default='trove.extensions.common.service.DefaultRootController',
                help='Root controller implementation for percona.'),
@@ -738,22 +762,10 @@ pxc_opts = [
     cfg.IntOpt('usage_timeout', default=450,
                help='Maximum time (in seconds) to wait for a Guest to become '
                     'active.'),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.mysql_impl',
-               help='Namespace to load backup strategies from.'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.mysql_impl',
-               help='Namespace to load restore strategies from.'),
     cfg.BoolOpt('volume_support', default=True,
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
-    cfg.DictOpt('backup_incremental_strategy',
-                default={'InnoBackupEx': 'InnoBackupExIncremental'},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental backup, the runner will use the default full '
-                'backup.'),
     cfg.ListOpt('ignore_users', default=['os_admin', 'root', 'clusterrepuser'],
                 help='Users to exclude when listing users.'),
     cfg.ListOpt('ignore_dbs',
@@ -817,12 +829,6 @@ redis_opts = [
                help='Default strategy to perform backups.',
                deprecated_name='backup_strategy',
                deprecated_group='DEFAULT'),
-    cfg.DictOpt('backup_incremental_strategy', default={},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental, the runner will use the default full backup.',
-                deprecated_name='backup_incremental_strategy',
-                deprecated_group='DEFAULT'),
     cfg.StrOpt('replication_strategy', default='RedisSyncReplication',
                help='Default strategy for replication.'),
     cfg.StrOpt('replication_namespace',
@@ -836,18 +842,6 @@ redis_opts = [
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
-    cfg.StrOpt('backup_namespace',
-               default="trove.guestagent.strategies.backup.experimental."
-                       "redis_impl",
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('restore_namespace',
-               default="trove.guestagent.strategies.restore.experimental."
-                       "redis_impl",
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
     cfg.BoolOpt('cluster_support', default=True,
                 help='Enable clusters to be created and managed.'),
     cfg.StrOpt('api_strategy',
@@ -892,12 +886,6 @@ cassandra_opts = [
                 help='List of UDP ports and/or port ranges to open '
                      'in the security group (only applicable '
                      'if trove_security_groups_support is True).'),
-    cfg.DictOpt('backup_incremental_strategy', default={},
-                help='Incremental strategy based on the default backup '
-                'strategy. For strategies that do not implement incremental '
-                'backups, the runner performs full backup instead.',
-                deprecated_name='backup_incremental_strategy',
-                deprecated_group='DEFAULT'),
     cfg.StrOpt('backup_strategy', default="NodetoolSnapshot",
                help='Default strategy to perform backups.',
                deprecated_name='backup_strategy',
@@ -911,18 +899,6 @@ cassandra_opts = [
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
-    cfg.StrOpt('backup_namespace',
-               default="trove.guestagent.strategies.backup.experimental."
-               "cassandra_impl",
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('restore_namespace',
-               default="trove.guestagent.strategies.restore.experimental."
-               "cassandra_impl",
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
     cfg.StrOpt('root_controller',
                default='trove.extensions.common.service.DefaultRootController',
                help='Root controller implementation for Cassandra.'),
@@ -1001,12 +977,6 @@ couchbase_opts = [
                help='Default strategy to perform backups.',
                deprecated_name='backup_strategy',
                deprecated_group='DEFAULT'),
-    cfg.DictOpt('backup_incremental_strategy', default={},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental, the runner will use the default full backup.',
-                deprecated_name='backup_incremental_strategy',
-                deprecated_group='DEFAULT'),
     cfg.StrOpt('replication_strategy', default=None,
                help='Default strategy for replication.'),
     cfg.StrOpt('mount_point', default='/var/lib/couchbase',
@@ -1017,18 +987,6 @@ couchbase_opts = [
                 'service during instance-create. The generated password for '
                 'the root user is immediately returned in the response of '
                 "instance-create as the 'password' field."),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.experimental.'
-               'couchbase_impl',
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.experimental.'
-               'couchbase_impl',
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
     cfg.BoolOpt('volume_support', default=True,
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
@@ -1065,12 +1023,6 @@ mongodb_opts = [
                help='Default strategy to perform backups.',
                deprecated_name='backup_strategy',
                deprecated_group='DEFAULT'),
-    cfg.DictOpt('backup_incremental_strategy', default={},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental, the runner will use the default full backup.',
-                deprecated_name='backup_incremental_strategy',
-                deprecated_group='DEFAULT'),
     cfg.StrOpt('replication_strategy', default=None,
                help='Default strategy for replication.'),
     cfg.StrOpt('mount_point', default='/var/lib/mongodb',
@@ -1108,18 +1060,6 @@ mongodb_opts = [
                'mongodb.guestagent.MongoDbGuestAgentStrategy',
                help='Class that implements datastore-specific Guest Agent API '
                     'logic.'),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.experimental.'
-                       'mongo_impl',
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.experimental.'
-                       'mongo_impl',
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
     cfg.PortOpt('mongodb_port', default=27017,
                 help='Port for mongod and mongos instances.'),
     cfg.PortOpt('configsvr_port', default=27019,
@@ -1148,6 +1088,15 @@ postgresql_group = cfg.OptGroup(
     'postgresql', title='PostgreSQL options',
     help="Oslo option group for the PostgreSQL datastore.")
 postgresql_opts = [
+    cfg.StrOpt(
+        'docker_image', default='postgres',
+        help='Database docker image.'
+    ),
+    cfg.StrOpt(
+        'backup_docker_image',
+        default='openstacktrove/db-backup-postgresql:1.1.0',
+        help='The docker image used for backup and restore.'
+    ),
     cfg.BoolOpt('icmp', default=False,
                 help='Whether to permit ICMP.',
                 deprecated_for_removal=True),
@@ -1161,20 +1110,18 @@ postgresql_opts = [
                      'if trove_security_groups_support is True).'),
     cfg.PortOpt('postgresql_port', default=5432,
                 help='The TCP port the server listens on.'),
-    cfg.StrOpt('backup_strategy', default='PgBaseBackup',
+    cfg.StrOpt('backup_strategy', default='pg_basebackup',
                help='Default strategy to perform backups.'),
-    cfg.DictOpt('backup_incremental_strategy',
-                default={'PgBaseBackup': 'PgBaseBackupIncremental'},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental, the runner will use the default full backup.'),
-    cfg.StrOpt('replication_strategy',
-               default='PostgresqlReplicationStreaming',
-               help='Default strategy for replication.'),
-    cfg.StrOpt('replication_namespace',
-               default='trove.guestagent.strategies.replication.experimental.'
-                       'postgresql_impl',
-               help='Namespace to load replication strategies from.'),
+    cfg.StrOpt(
+        'replication_strategy',
+        default='PostgresqlReplicationStreaming',
+        help='Default strategy for replication.'
+    ),
+    cfg.StrOpt(
+        'replication_namespace',
+        default='trove.guestagent.strategies.replication.postgresql',
+        help='Namespace to load replication strategies from.'
+    ),
     cfg.StrOpt('mount_point', default='/var/lib/postgresql',
                help="Filesystem path for mounting "
                "volumes if volume support is enabled."),
@@ -1187,18 +1134,10 @@ postgresql_opts = [
                 'service during instance-create. The generated password for '
                 'the root user is immediately returned in the response of '
                 "instance-create as the 'password' field."),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.experimental.'
-               'postgresql_impl',
-               help='Namespace to load backup strategies from.'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.experimental.'
-               'postgresql_impl',
-               help='Namespace to load restore strategies from.'),
     cfg.BoolOpt('volume_support', default=True,
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb'),
-    cfg.ListOpt('ignore_users', default=['os_admin', 'postgres', 'root']),
+    cfg.ListOpt('ignore_users', default=['os_admin', 'postgres']),
     cfg.ListOpt('ignore_dbs', default=['os_admin', 'postgres']),
     cfg.StrOpt('root_controller',
                default='trove.extensions.common.service.DefaultRootController',
@@ -1247,16 +1186,6 @@ couchdb_opts = [
                help='Default strategy to perform backups.'),
     cfg.StrOpt('replication_strategy', default=None,
                help='Default strategy for replication.'),
-    cfg.StrOpt('backup_namespace', default='trove.guestagent.strategies'
-               '.backup.experimental.couchdb_impl',
-               help='Namespace to load backup strategies from.'),
-    cfg.StrOpt('restore_namespace', default='trove.guestagent.strategies'
-               '.restore.experimental.couchdb_impl',
-               help='Namespace to load restore strategies from.'),
-    cfg.DictOpt('backup_incremental_strategy', default={},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental, the runner will use the default full backup.'),
     cfg.BoolOpt('root_on_create', default=False,
                 help='Enable the automatic creation of the root user for the '
                 'service during instance-create. The generated password for '
@@ -1302,10 +1231,6 @@ vertica_opts = [
                      'if trove_security_groups_support is True).'),
     cfg.StrOpt('backup_strategy', default=None,
                help='Default strategy to perform backups.'),
-    cfg.DictOpt('backup_incremental_strategy', default={},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental, the runner will use the default full backup.'),
     cfg.StrOpt('replication_strategy', default=None,
                help='Default strategy for replication.'),
     cfg.StrOpt('mount_point', default='/var/lib/vertica',
@@ -1316,9 +1241,11 @@ vertica_opts = [
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
     cfg.StrOpt('backup_namespace', default=None,
-               help='Namespace to load backup strategies from.'),
+               help='Namespace to load backup strategies from.',
+               deprecated_for_removal=True),
     cfg.StrOpt('restore_namespace', default=None,
-               help='Namespace to load restore strategies from.'),
+               help='Namespace to load restore strategies from.',
+               deprecated_for_removal=True),
     cfg.IntOpt('readahead_size', default=2048,
                help='Size(MB) to be set as readahead_size for data volume'),
     cfg.BoolOpt('cluster_support', default=True,
@@ -1386,22 +1313,6 @@ db2_opts = [
                 'service during instance-create. The generated password for '
                 'the root user is immediately returned in the response of '
                 "instance-create as the 'password' field."),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.experimental.'
-                       'db2_impl',
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.experimental.'
-                       'db2_impl',
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.DictOpt('backup_incremental_strategy', default={},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental, the runner will use the default full backup.'),
     cfg.ListOpt('ignore_users', default=['PUBLIC', 'DB2INST1']),
     cfg.StrOpt('root_controller',
                default='trove.extensions.common.service.DefaultRootController',
@@ -1431,21 +1342,14 @@ mariadb_opts = [
                 help='List of UDP ports and/or port ranges to open '
                      'in the security group (only applicable '
                      'if trove_security_groups_support is True).'),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.experimental'
-                       '.mariadb_impl',
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('backup_strategy', default='MariaBackup',
+    cfg.StrOpt('backup_strategy', default='mariabackup',
                help='Default strategy to perform backups.',
                deprecated_name='backup_strategy',
                deprecated_group='DEFAULT'),
     cfg.StrOpt('replication_strategy', default='MariaDBGTIDReplication',
                help='Default strategy for replication.'),
     cfg.StrOpt('replication_namespace',
-               default='trove.guestagent.strategies.replication.experimental'
-               '.mariadb_gtid',
+               default='trove.guestagent.strategies.replication.mariadb_gtid',
                help='Namespace to load replication strategies from.'),
     cfg.StrOpt('mount_point', default='/var/lib/mysql',
                help="Filesystem path for mounting "
@@ -1458,25 +1362,10 @@ mariadb_opts = [
     cfg.IntOpt('usage_timeout', default=400,
                help='Maximum time (in seconds) to wait for a Guest to become '
                     'active.'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.experimental'
-                       '.mariadb_impl',
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
     cfg.BoolOpt('volume_support', default=True,
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
-    cfg.DictOpt('backup_incremental_strategy',
-                default={'MariaBackup':
-                         'MariaBackupIncremental'},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental backup, the runner will use the default full '
-                'backup.',
-                deprecated_name='backup_incremental_strategy',
-                deprecated_group='DEFAULT'),
     cfg.StrOpt('root_controller',
                default='trove.extensions.common.service.DefaultRootController',
                help='Root controller implementation for mysql.'),
@@ -1520,6 +1409,15 @@ mariadb_opts = [
                help='Character length of generated passwords.',
                deprecated_name='default_password_length',
                deprecated_group='DEFAULT'),
+    cfg.StrOpt(
+        'docker_image', default='mariadb',
+        help='Database docker image.'
+    ),
+    cfg.StrOpt(
+        'backup_docker_image',
+        default='openstacktrove/db-backup-mariadb:1.1.0',
+        help='The docker image used for backup and restore.'
+    ),
 ]
 
 # RPC version groups
@@ -1553,7 +1451,14 @@ network_opts = [
         help='ID of the Neutron public network to create floating IP for the '
              'public trove instance. If not given, Trove will try to query '
              'all the public networks and use the first one in the list.'
-    )
+    ),
+    cfg.BoolOpt(
+        'enable_access_check', default=True,
+        help='Check if the user provided network is associated with router. '
+             'This is needed for the instance initialization. The check is '
+             'also necessary when creating public facing instance. A scenario '
+             'to set this option False is when using Neutron provider '
+             'network.')
 ]
 
 service_credentials_group = cfg.OptGroup(
@@ -1595,6 +1500,24 @@ service_credentials_opts = [
                help="Keystone region name of the Trove service project."),
 ]
 
+guest_agent_group = cfg.OptGroup(
+    'guest_agent', title='Guest Agent options',
+    help="Config options used by guest agent.")
+guest_agent_opts = [
+    cfg.StrOpt(
+        'container_registry',
+        help='URL to the registry. E.g. https://index.docker.io/v1/'
+    ),
+    cfg.StrOpt(
+        'container_registry_username',
+        help='The registry username.'
+    ),
+    cfg.StrOpt(
+        'container_registry_password',
+        help='The plaintext registry password.'
+    ),
+]
+
 CONF = cfg.CONF
 
 CONF.register_opts(path_opts)
@@ -1617,6 +1540,7 @@ CONF.register_group(db2_group)
 CONF.register_group(mariadb_group)
 CONF.register_group(network_group)
 CONF.register_group(service_credentials_group)
+CONF.register_group(guest_agent_group)
 
 CONF.register_opts(mysql_opts, mysql_group)
 CONF.register_opts(percona_opts, percona_group)
@@ -1632,6 +1556,7 @@ CONF.register_opts(db2_opts, db2_group)
 CONF.register_opts(mariadb_opts, mariadb_group)
 CONF.register_opts(network_opts, network_group)
 CONF.register_opts(service_credentials_opts, service_credentials_group)
+CONF.register_opts(guest_agent_opts, guest_agent_group)
 
 CONF.register_opts(rpcapi_cap_opts, upgrade_levels)
 
@@ -1639,6 +1564,25 @@ profiler.set_defaults(CONF)
 logging.register_options(CONF)
 
 
+def list_opts():
+    keystone_middleware_opts = auth_token.list_opts()
+    keystone_loading_opts = [(
+        'keystone_authtoken', loading.get_auth_plugin_conf_options('password')
+    )]
+
+    trove_opts = [
+        (None, path_opts + versions_opts + common_opts),
+        ('database', database_opts),
+        (mysql_group, mysql_opts),
+        (mariadb_group, mariadb_opts),
+        (network_group, network_opts),
+        (service_credentials_group, service_credentials_opts),
+        (guest_agent_group, guest_agent_opts),
+    ]
+
+    return keystone_middleware_opts + keystone_loading_opts + trove_opts
+
+
 def custom_parser(parsername, parser):
     CONF.register_cli_opt(cfg.SubCommandOpt(parsername, handler=parser))
 
