@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from functools import reduce
 import inspect
 import operator
 import os
@@ -21,13 +22,12 @@ import re
 import stat
 import tempfile
 
-from functools import reduce
 from oslo_concurrency.processutils import UnknownArgumentError
 
 from trove.common import exception
+from trove.common import utils
 from trove.common.i18n import _
 from trove.common.stream_codecs import IdentityCodec
-from trove.common import utils
 
 REDHAT = 'redhat'
 DEBIAN = 'debian'
@@ -143,6 +143,7 @@ def _read_file_as_root(path, open_flag, convert_func):
 
 def write_file(path, data, codec=IdentityCodec(), as_root=False, encode=True):
     """Write data into file using a given codec.
+
     Overwrite any existing contents.
     The written file can be read back into its original
     form by 'read_file'.
@@ -479,11 +480,46 @@ def service_discovery(service_candidates):
     return result
 
 
-def create_directory(dir_path, user=None, group=None, force=True, **kwargs):
+def execute_shell_cmd(cmd, options, *args, **kwargs):
+    """Execute a given shell command passing it
+    given options (flags) and arguments.
+
+    Takes optional keyword arguments:
+    :param as_root:        Execute as root.
+    :type as_root:         boolean
+
+    :param timeout:        Number of seconds if specified,
+                           default if not.
+                           There is no timeout if set to None.
+    :type timeout:         integer
+
+    :raises:               class:`UnknownArgumentError` if passed unknown args.
+    """
+
+    exec_args = {}
+    if kwargs.pop('as_root', False):
+        exec_args['run_as_root'] = True
+        exec_args['root_helper'] = 'sudo'
+
+    if 'timeout' in kwargs:
+        exec_args['timeout'] = kwargs.pop('timeout')
+
+    exec_args['shell'] = kwargs.pop('shell', False)
+
+    if kwargs:
+        raise UnknownArgumentError(_("Got unknown keyword args: %r") % kwargs)
+
+    cmd_flags = _build_command_options(options)
+    cmd_args = cmd_flags + list(args)
+    stdout, stderr = utils.execute_with_timeout(cmd, *cmd_args, **exec_args)
+    return stdout
+
+
+def ensure_directory(dir_path, user=None, group=None, force=True, **kwargs):
     """Create a given directory and update its ownership
     (recursively) to the given user and group if any.
 
-    seealso:: _execute_shell_cmd for valid optional keyword arguments.
+    seealso:: execute_shell_cmd for valid optional keyword arguments.
 
     :param dir_path:        Path to the created directory.
     :type dir_path:         string
@@ -513,7 +549,7 @@ def create_directory(dir_path, user=None, group=None, force=True, **kwargs):
 def chown(path, user, group, recursive=True, force=False, **kwargs):
     """Changes the owner and group of a given file.
 
-    seealso:: _execute_shell_cmd for valid optional keyword arguments.
+    seealso:: execute_shell_cmd for valid optional keyword arguments.
 
     :param path:         Path to the modified file.
     :type path:          string
@@ -543,7 +579,7 @@ def chown(path, user, group, recursive=True, force=False, **kwargs):
 
     owner_group_modifier = _build_user_group_pair(user, group)
     options = (('f', force), ('R', recursive))
-    _execute_shell_cmd('chown', options, owner_group_modifier, path, **kwargs)
+    execute_shell_cmd('chown', options, owner_group_modifier, path, **kwargs)
 
 
 def _build_user_group_pair(user, group):
@@ -559,17 +595,18 @@ def _create_directory(dir_path, force=True, **kwargs):
     :param force:           No error if existing, make parent directories
                             as needed.
     :type force:            boolean
+    :param as_root: Run as root user, default: False.
     """
 
     options = (('p', force),)
-    _execute_shell_cmd('mkdir', options, dir_path, **kwargs)
+    execute_shell_cmd('mkdir', options, dir_path, **kwargs)
 
 
 def chmod(path, mode, recursive=True, force=False, **kwargs):
     """Changes the mode of a given file.
 
     :seealso: Modes for more information on the representation of modes.
-    :seealso: _execute_shell_cmd for valid optional keyword arguments.
+    :seealso: execute_shell_cmd for valid optional keyword arguments.
 
     :param path:            Path to the modified file.
     :type path:             string
@@ -592,7 +629,7 @@ def chmod(path, mode, recursive=True, force=False, **kwargs):
     if path:
         options = (('f', force), ('R', recursive))
         shell_modes = _build_shell_chmod_mode(mode)
-        _execute_shell_cmd('chmod', options, shell_modes, path, **kwargs)
+        execute_shell_cmd('chmod', options, shell_modes, path, **kwargs)
     else:
         raise exception.UnprocessableEntity(
             _("Cannot change mode of a blank file."))
@@ -602,7 +639,7 @@ def change_user_group(user, group, append=True, add_group=True, **kwargs):
     """Adds a user to groups by using the usermod linux command with -a and
     -G options.
 
-    seealso:: _execute_shell_cmd for valid optional keyword arguments.
+    seealso:: execute_shell_cmd for valid optional keyword arguments.
 
     :param user:            Username.
     :type user:             string
@@ -631,7 +668,7 @@ def change_user_group(user, group, append=True, add_group=True, **kwargs):
         raise exception.UnprocessableEntity(_("Missing group."))
 
     options = (('a', append), ('G', add_group))
-    _execute_shell_cmd('usermod', options, group, user, **kwargs)
+    execute_shell_cmd('usermod', options, group, user, **kwargs)
 
 
 def _build_shell_chmod_mode(mode):
@@ -667,7 +704,7 @@ def _build_shell_chmod_mode(mode):
 def remove(path, force=False, recursive=True, **kwargs):
     """Remove a given file or directory.
 
-    :seealso: _execute_shell_cmd for valid optional keyword arguments.
+    :seealso: execute_shell_cmd for valid optional keyword arguments.
 
     :param path:            Path to the removed file.
     :type path:             string
@@ -683,7 +720,7 @@ def remove(path, force=False, recursive=True, **kwargs):
 
     if path:
         options = (('f', force), ('R', recursive))
-        _execute_shell_cmd('rm', options, path, **kwargs)
+        execute_shell_cmd('rm', options, path, **kwargs)
     else:
         raise exception.UnprocessableEntity(_("Cannot remove a blank file."))
 
@@ -693,7 +730,7 @@ def move(source, destination, force=False, **kwargs):
     Move attempts to preserve the original ownership, permissions and
     timestamps.
 
-    :seealso: _execute_shell_cmd for valid optional keyword arguments.
+    :seealso: execute_shell_cmd for valid optional keyword arguments.
 
     :param source:          Path to the source location.
     :type source:           string
@@ -714,16 +751,17 @@ def move(source, destination, force=False, **kwargs):
         raise exception.UnprocessableEntity(_("Missing destination path."))
 
     options = (('f', force),)
-    _execute_shell_cmd('mv', options, source, destination, **kwargs)
+    execute_shell_cmd('mv', options, source, destination, **kwargs)
 
 
 def copy(source, destination, force=False, preserve=False, recursive=True,
          dereference=False, **kwargs):
     """Copy a given file or directory to another location.
+
     Copy does NOT attempt to preserve ownership, permissions and timestamps
     unless the 'preserve' option is enabled.
 
-    :seealso: _execute_shell_cmd for valid optional keyword arguments.
+    :seealso: execute_shell_cmd for valid optional keyword arguments.
 
     :param source:          Path to the source location.
     :type source:           string
@@ -755,7 +793,7 @@ def copy(source, destination, force=False, preserve=False, recursive=True,
 
     options = (('f', force), ('p', preserve), ('R', recursive),
                ('L', dereference))
-    _execute_shell_cmd('cp', options, source, destination, **kwargs)
+    execute_shell_cmd('cp', options, source, destination, **kwargs)
 
 
 def get_bytes_free_on_fs(path):
@@ -792,7 +830,7 @@ def list_files_in_directory(root_dir, recursive=False, pattern=None,
         if pattern:
             cmd_args.extend(['-regextype', 'posix-extended',
                              '-regex', os.path.join('.*', pattern) + '$'])
-        files = _execute_shell_cmd('find', [], *cmd_args, as_root=True)
+        files = execute_shell_cmd('find', [], *cmd_args, as_root=True)
         return {fp for fp in files.splitlines()}
 
     return {os.path.abspath(os.path.join(root, name))
@@ -802,39 +840,6 @@ def list_files_in_directory(root_dir, recursive=False, pattern=None,
             if not pattern or re.match(pattern, name)}
 
 
-def _execute_shell_cmd(cmd, options, *args, **kwargs):
-    """Execute a given shell command passing it
-    given options (flags) and arguments.
-
-    Takes optional keyword arguments:
-    :param as_root:        Execute as root.
-    :type as_root:         boolean
-
-    :param timeout:        Number of seconds if specified,
-                           default if not.
-                           There is no timeout if set to None.
-    :type timeout:         integer
-
-    :raises:               class:`UnknownArgumentError` if passed unknown args.
-    """
-
-    exec_args = {}
-    if kwargs.pop('as_root', False):
-        exec_args['run_as_root'] = True
-        exec_args['root_helper'] = 'sudo'
-
-    if 'timeout' in kwargs:
-        exec_args['timeout'] = kwargs.pop('timeout')
-
-    if kwargs:
-        raise UnknownArgumentError(_("Got unknown keyword args: %r") % kwargs)
-
-    cmd_flags = _build_command_options(options)
-    cmd_args = cmd_flags + list(args)
-    stdout, stderr = utils.execute_with_timeout(cmd, *cmd_args, **exec_args)
-    return stdout
-
-
 def _build_command_options(options):
     """Build a list of flags from given pairs (option, is_enabled).
     Each option is prefixed with a single '-'.
@@ -846,7 +851,7 @@ def _build_command_options(options):
 
 def get_device(path, as_root=False):
     """Get the device that a given path exists on."""
-    stdout = _execute_shell_cmd('df', [], path, as_root=as_root)
+    stdout = execute_shell_cmd('df', [], path, as_root=as_root)
     return stdout.splitlines()[1].split()[0]
 
 
@@ -867,3 +872,35 @@ def is_mount(path):
 def get_current_user():
     """Returns name of the current OS user"""
     return pwd.getpwuid(os.getuid())[0]
+
+
+def create_user(user_name, user_id, group_name=None, group_id=None):
+    group_name = group_name or user_name
+    group_id = group_id or user_id
+
+    try:
+        execute_shell_cmd('groupadd', [], '--gid', group_id, group_name,
+                          as_root=True)
+    except exception.ProcessExecutionError as err:
+        if 'already exists' not in err.stderr:
+            raise exception.UnprocessableEntity(
+                'Failed to add group %s, error: %s' % (group_name, err.stderr)
+            )
+
+    try:
+        execute_shell_cmd('useradd', [], '--uid', user_id, '--gid', group_id,
+                          '-M', user_name, as_root=True)
+    except exception.ProcessExecutionError as err:
+        if 'already exists' not in err.stderr:
+            raise exception.UnprocessableEntity(
+                'Failed to add user %s, error: %s' % (user_name, err.stderr)
+            )
+
+
+def remove_dir_contents(folder):
+    """Remove all the files and sub-directories but keep the folder.
+
+    Use shell=True here because shell=False doesn't support '*'
+    """
+    path = os.path.join(folder, '*')
+    execute_shell_cmd(f'rm -rf {path}', [], shell=True, as_root=True)
