@@ -19,7 +19,6 @@ import json
 import netaddr
 import os
 import proboscis
-import six
 import sys
 import time as timer
 import types
@@ -284,13 +283,12 @@ class LogOnFail(type):
 
                 # Only report on the first error that occurs
                 mcs.reset_inst_ids()
-                six.reraise(extype, exvalue, extb)
+                raise exvalue.with_traceback(extb)
 
         return wrapper
 
 
-@six.add_metaclass(LogOnFail)
-class TestRunner(object):
+class TestRunner(object, metaclass=LogOnFail):
 
     """
     Base class for all 'Runner' classes.
@@ -327,7 +325,7 @@ class TestRunner(object):
     instance_info = InstanceTestInfo()
     report = CONFIG.get_report()
 
-    def __init__(self, sleep_time=10, timeout=1800):
+    def __init__(self, sleep_time=10, timeout=900):
         self.def_sleep_time = sleep_time
         self.def_timeout = timeout
 
@@ -606,6 +604,19 @@ class TestRunner(object):
             self.assert_equal(expected_http_code, client.last_http_code,
                               "Unexpected client status code")
 
+    def assert_instance_operating_status(self, instance_id, expected_status):
+        self.report.log(f"Waiting for expected_status ({expected_status}) "
+                        f"for instances: {instance_id}")
+
+        def wait_for_operating_status():
+            instance = self.get_instance(instance_id, self.admin_client)
+            if instance.operating_status == expected_status:
+                return True
+            return False
+
+        poll_until(wait_for_operating_status, sleep_time=self.def_sleep_time,
+                   time_out=self.def_timeout)
+
     def assert_all_instance_states(self, instance_ids, expected_states,
                                    fast_fail_status=None,
                                    require_all_states=False):
@@ -649,6 +660,15 @@ class TestRunner(object):
         self.report.log("Waiting for states (%s) for instance: %s" %
                         (expected_states, instance_id))
 
+        # Replace HEALTHY with ACTIVE. This is needed after operating_status
+        # is introduced in Trove.
+        wait_operating_status = False
+        if 'HEALTHY' in expected_states:
+            wait_operating_status = True
+            expected_states.remove('HEALTHY')
+            if 'ACTIVE' not in expected_states:
+                expected_states.append('ACTIVE')
+
         if fast_fail_status is None:
             fast_fail_status = ['ERROR', 'FAILED']
         found = False
@@ -679,6 +699,9 @@ class TestRunner(object):
                     "Instance state was not '%s', moving to the next expected "
                     "state." % status)
 
+        if found and wait_operating_status:
+            self.assert_instance_operating_status(instance_id, 'HEALTHY')
+
         return found
 
     def _time_since(self, start_time):
@@ -985,7 +1008,7 @@ class CheckInstance(AttrCheck):
         if 'flavor' not in self.instance:
             self.fail("'flavor' not found in instance.")
         else:
-            allowed_attrs = ['id']
+            allowed_attrs = ['id', 'links']
             self.contains_allowed_attrs(
                 self.instance['flavor'], allowed_attrs,
                 msg="Flavor")
@@ -994,7 +1017,7 @@ class CheckInstance(AttrCheck):
         if 'datastore' not in self.instance:
             self.fail("'datastore' not found in instance.")
         else:
-            allowed_attrs = ['type', 'version']
+            allowed_attrs = ['type', 'version', 'version_number']
             self.contains_allowed_attrs(
                 self.instance['datastore'], allowed_attrs,
                 msg="datastore")
