@@ -79,17 +79,19 @@ class fake_Server(object):
         self.flavor_id = None
         self.files = None
         self.userdata = None
+        self.meta = None
         self.block_device_mapping_v2 = None
         self.status = 'HEALTHY'
         self.key_name = None
 
 
 class fake_ServerManager(object):
+
     def create(self, name, image_id, flavor_id, files, userdata,
                block_device_mapping_v2=None,
                availability_zone=None,
                nics=None, config_drive=False,
-               scheduler_hints=None, key_name=None):
+               scheduler_hints=None, key_name=None, meta=None):
         server = fake_Server()
         server.id = "server_id"
         server.name = name
@@ -101,7 +103,7 @@ class fake_ServerManager(object):
         server.availability_zone = availability_zone
         server.nics = nics
         server.key_name = key_name
-
+        server.meta = meta
         return server
 
 
@@ -210,7 +212,9 @@ class BaseFreshInstanceTasksTest(trove_testtools.TestCase):
             self.guestconfig = f.name
             f.write(self.guestconfig_content)
         self.freshinstancetasks = taskmanager_models.FreshInstanceTasks(
-            None, Mock(), None, None)
+            None, MagicMock(), None, None)
+        self.freshinstancetasks.context = trove.common.context.TroveContext(
+            user='test_user')
 
     def tearDown(self):
         super(BaseFreshInstanceTasksTest, self).tearDown()
@@ -306,6 +310,9 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
         mock_servers_create = mock_nova_client.servers.create
         self.freshinstancetasks._create_server('fake-flavor', 'fake-image',
                                                'mysql', None, None, None)
+        meta = {'trove_project_id': self.freshinstancetasks.tenant_id,
+                'trove_user_id': 'test_user',
+                'trove_instance_id': self.freshinstancetasks.id}
         mock_servers_create.assert_called_with(
             'fake-hostname', 'fake-image',
             'fake-flavor', files={},
@@ -315,7 +322,8 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
             nics=None,
             config_drive=True,
             scheduler_hints=None,
-            key_name=None
+            key_name=None,
+            meta=meta,
         )
 
     @patch.object(InstanceServiceStatus, 'find_by',
@@ -378,7 +386,7 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
             is_public=False
         )
         mock_build_volume_info.assert_called_with(
-            'mysql', volume_size=2,
+            'mysql', availability_zone=None, volume_size=2,
             volume_type='volume_type'
         )
         mock_guest_prepare.assert_called_with(
@@ -411,7 +419,7 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
                                             *args):
         self.patch_conf_property('management_networks', ['fake-mgmt-uuid'])
 
-        mock_client = Mock()
+        mock_client = MagicMock()
         mock_client.create_security_group.return_value = {
             'security_group': {'id': 'fake-sg-id'}
         }
@@ -449,7 +457,7 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
         )
 
         mock_build_volume_info.assert_called_with(
-            'mysql', volume_size=2,
+            'mysql', availability_zone=None, volume_size=2,
             volume_type='volume_type'
         )
         mock_guest_prepare.assert_called_with(
@@ -468,6 +476,7 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
             "floatingip": {
                 'floating_network_id': 'fake-public-net-id',
                 'port_id': 'fake-user-port-id',
+                'project_id': mock.ANY
             }
         }
         mock_client.create_floatingip.assert_called_once_with(
@@ -1028,10 +1037,8 @@ class BackupTasksTest(trove_testtools.TestCase):
 
         self.assertTrue(self.backup.deleted)
 
-    @patch('trove.taskmanager.models.LOG')
     @patch('trove.common.clients.create_swift_client')
-    def test_delete_backup_fail_delete_manifest(self, mock_swift_client,
-                                                mock_logging):
+    def test_delete_backup_fail_delete_manifest(self, mock_swift_client):
         client_mock = MagicMock()
         client_mock.head_object.return_value = {}
         client_mock.delete_object.side_effect = ClientException("foo")
@@ -1061,6 +1068,12 @@ class BackupTasksTest(trove_testtools.TestCase):
         client_mock.delete_object.assert_called_once_with('container',
                                                           '12e48.xbstream.gz')
 
+    def test_delete_backup_restored(self):
+        self.backup.state = state.BackupState.RESTORED
+        taskmanager_models.BackupTasks.delete_backup(mock.ANY, self.backup.id)
+
+        self.assertTrue(self.backup.deleted)
+
     def test_parse_manifest(self):
         manifest = 'container/prefix'
         cont, prefix = taskmanager_models.BackupTasks._parse_manifest(manifest)
